/**
 * EasyBeans
 * Copyright (C) 2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestNGBundleTracker.java 6135 2012-01-24 17:02:48Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.testng4osgi.tracker;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.wiring.BundleCapability;
import org.osgi.framework.wiring.BundleRevision;
import org.osgi.framework.wiring.BundleWire;
import org.osgi.framework.wiring.BundleWiring;
import org.osgi.util.tracker.BundleTrackerCustomizer;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.testng4osgi.launcher.TestNGLauncher;

/**
 * Bundle tracker that will track any TestNG tests
 * @author benoitf
 *
 */
public class TestNGBundleTracker implements BundleTrackerCustomizer {
    
    /**
     * TestNG annotation package.
     */
    private static final String TESTNG_ANNOTATION_PACKAGE = "org.testng.annotations";

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(TestNGBundleTracker.class);
    
    /**
     * Launcher that needs to be used when adding new tests.
     */
    private TestNGLauncher testNGLauncher;
    
    /**
     * Build a new tracker by using the given launcher.
     * @param testNGLauncher the given launcher.
     */
    public TestNGBundleTracker(TestNGLauncher testNGLauncher) {
        this.testNGLauncher = testNGLauncher;
    }
    
    /**
     * Called when a new bundle is coming on the platform.
     * @param bundle the bundle to inspect
     * @param event the bundle event
     */
    public Object addingBundle(Bundle bundle, BundleEvent event) {
        
        // Adapt the bundle 
        BundleWiring bundleWiring = bundle.adapt(BundleWiring.class);
        
        // Import packages ?
        List<BundleWire> importPackages = bundleWiring.getRequiredWires(BundleRevision.PACKAGE_NAMESPACE);
        if (importPackages == null) {
            return null;
        }
        
        // Search testng annotation
        //TODO : Search also something like META-INF/testng.xml
        boolean found = false;
        int i = 0;
        while (!found && i < importPackages.size()) {
            BundleWire bundleWire = importPackages.get(i);

            // Now, search if the bundle capability (which is providing the package is providing the testng import)
            BundleCapability bundleCapability = bundleWire.getCapability();
            if (bundleCapability != null) {
                Map<String, Object> attributes = bundleCapability.getAttributes();
                if (attributes != null) {
                    // Do we have testNG imports ?
                    Object packageName = attributes.get(BundleRevision.PACKAGE_NAMESPACE);
                    if (TESTNG_ANNOTATION_PACKAGE.equals(packageName)) {
                        found = true;
                    }
                }
            }
            i++;
        }
        
        // Not found, skip
        if (!found) {
            return null;
        }
        

        // TestNG has been found, needs to scan all the classes
        LOGGER.debug("The TestNG import has been found in the bundle ''{0}'' with id ''{1}''", bundle.getSymbolicName(), bundle.getBundleId());
        List<URL> classEntriesURL = bundleWiring.findEntries("/", "*.class", BundleWiring.FINDENTRIES_RECURSE);
        if (classEntriesURL == null) {
            return null;
        }
            
        List<Class<?>> classes = new ArrayList<Class<?>>();
        for (URL url : classEntriesURL) {

            // Extract classname from path which is like "/org/ow2/easybeans/tests/osgi/dummy/DummyTestSuite.class"
            String path = url.getPath();
            String className = null;
            if (path != null) {
                className = path.substring(1).replace("/", ".");
                if (className.endsWith(".class")) {
                    className = className.substring(0, className.length() - 6);
                } else {
                    LOGGER.warn("Invalid class named ''{0}''", className);
                    className = null;
                }
            }
            if (className != null) {
                try {
                    Class<?> clazz = bundleWiring.getClassLoader().loadClass(className);
                    classes.add(clazz);
                } catch (ClassNotFoundException e) {
                    LOGGER.error("Unable to load class ''{0}''", className, e);
                }
            }
        }

        
        // Add classes to the testng launcher
        if (!classes.isEmpty()) {
            LOGGER.debug("classes = {0}", classes);
            testNGLauncher.addTests(bundle.getBundleContext(), classes);
        }
            
        //FIXME ? notifiying object to keep ?
        return null;
    }

    /**
     * FIXME: update/remove tests
     */
    public void modifiedBundle(Bundle bundle, BundleEvent event, Object object) {
        
    }

    /**
     * FIXME: remove tests
     */
    public void removedBundle(Bundle bundle, BundleEvent event, Object object) {
        
        
    }

}
